const path = require('path');
const fs = require('fs');

const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const GenerateJsonPlugin = require('generate-json-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const ScriptExtHtmlWebpackPlugin = require('script-ext-html-webpack-plugin');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');

const config = require('./config').configBuilder;
const isProduction = process.env.NODE_ENV === 'production';

const defines = {
  __VERSION__: JSON.stringify(require('./package.json').version)
};

const devServer = {
  host: process.env.HOST || '0.0.0.0',
  port: process.env.port || '8006',
  hot: true,
  open: false,
  disableHostCheck: true,
  watchOptions: {ignored: /node_modules/}
};

if (config.ssl && config.ssl.key && config.ssl.cert) {
  devServer.https = {
    key: fs.readFileSync(config.ssl.key),
    cert: fs.readFileSync(config.ssl.cert)
  };
}

module.exports = {
  mode: isProduction ? 'production' : 'development',
  devtool: 'source-map',
  entry: {
    'inbox': './src/set-inbox.ts'
  },
  output: {
    path: path.join(__dirname, 'dist'),
    filename: `pushwoosh-[name].${isProduction ? '' : 'uncompress.'}js`,
    globalObject: 'this'
  },
  resolve: {
    extensions: ['.ts', '.js', '.json'],
    modules: [path.resolve(__dirname, 'src'), 'node_modules']
  },
  module: {
    rules: [
      {
        test: /\.ts$/,
        use: 'awesome-typescript-loader'
      },
      {
        test: /\.css$/,
        use: ['to-string-loader', 'css-loader', 'postcss-loader']
      }
    ]
  },
  optimization: {
    minimizer: [
      new UglifyJsPlugin({
        uglifyOptions: {
          compress: {
            pure_getters: true,
            unsafe: true,
            unsafe_comps: true,
            warnings: false
          },
          mangle: true,
          output: {
            comments: false
          }
        }
      })
    ]
  },
  plugins: [
    new CleanWebpackPlugin([path.resolve(__dirname, 'dist')]),
    new webpack.optimize.OccurrenceOrderPlugin(),
    new webpack.DefinePlugin(defines),
    new HtmlWebpackPlugin({
      inject: false,
      template: 'static/index.html',
      externals: {
        initParams: JSON.stringify(config.initParams)
      },
      minify: false,
    }),
    new ScriptExtHtmlWebpackPlugin({async: /\.js$/}),
    new GenerateJsonPlugin('manifest.json', config.manifest),
    new CopyWebpackPlugin([
      {
        from: path.resolve(__dirname, './node_modules/web-push-notifications/lib/service-worker.js'),
        to: 'static',
        ignore: ['.*']
      }
    ]),

    !isProduction && new webpack.HotModuleReplacementPlugin()
  ].filter(x => x),

  devServer
};
